/*=========================================================================

  Module:    $RCSfile: vtkKWWidgetSetSubclass.h.in,v $

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkKWComboBoxSet - a concrete set of vtkKWComboBox
// .SECTION Description
// A composite widget to conveniently store, allocate, create and pack a 
// set of vtkKWComboBox. 
// Each vtkKWComboBox is created, removed or queried based
// on a unique ID provided by the user (ids are *not* handled by the class
// since it is likely that they will be defined as enum's or #define by
// the user for easier retrieval).
// As a subclass of vtkKWWidgetSet, it inherits methods to set the widgets
// visibility individually, set the layout parameters, and query each widget.
// Widgets are packed (gridded) in the order they were added.
// .SECTION See Also
// vtkKWWidgetSet vtkKWComboBox

#ifndef __vtkKWComboBoxSet_h
#define __vtkKWComboBoxSet_h

#include "vtkKWWidgetSet.h"

class vtkKWComboBox;

class KWWIDGETS_EXPORT vtkKWComboBoxSet : public vtkKWWidgetSet
{
public:
  static vtkKWComboBoxSet* New();
  vtkTypeRevisionMacro(vtkKWComboBoxSet,vtkKWWidgetSet);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Add a vtkKWComboBox to the set.
  // The id has to be unique among the set.
  // Return a pointer to the vtkKWComboBox, or NULL on error.
  virtual vtkKWComboBox* AddWidget(int id);

  // Description:
  // Get a vtkKWComboBox from the set, given its unique id.
  // Return a pointer to the vtkKWComboBox, or NULL on error.
  virtual vtkKWComboBox* GetWidget(int id);

protected:
  vtkKWComboBoxSet() {};
  ~vtkKWComboBoxSet() {};

  // Helper methods

  virtual vtkKWWidget* AllocateAndCreateWidget();

private:
  vtkKWComboBoxSet(const vtkKWComboBoxSet&); // Not implemented
  void operator=(const vtkKWComboBoxSet&); // Not implemented
};

#endif
