/*=========================================================================

  Module:    $RCSfile: vtkKWWidgetSetSubclass.h.in,v $

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkKWLabelWithLabelSet - a concrete set of vtkKWLabelWithLabel
// .SECTION Description
// A composite widget to conveniently store, allocate, create and pack a 
// set of vtkKWLabelWithLabel. 
// Each vtkKWLabelWithLabel is created, removed or queried based
// on a unique ID provided by the user (ids are *not* handled by the class
// since it is likely that they will be defined as enum's or #define by
// the user for easier retrieval).
// As a subclass of vtkKWWidgetSet, it inherits methods to set the widgets
// visibility individually, set the layout parameters, and query each widget.
// Widgets are packed (gridded) in the order they were added.
// .SECTION See Also
// vtkKWWidgetSet vtkKWLabelWithLabel

#ifndef __vtkKWLabelWithLabelSet_h
#define __vtkKWLabelWithLabelSet_h

#include "vtkKWWidgetSet.h"

class vtkKWLabelWithLabel;

class KWWIDGETS_EXPORT vtkKWLabelWithLabelSet : public vtkKWWidgetSet
{
public:
  static vtkKWLabelWithLabelSet* New();
  vtkTypeRevisionMacro(vtkKWLabelWithLabelSet,vtkKWWidgetSet);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Add a vtkKWLabelWithLabel to the set.
  // The id has to be unique among the set.
  // Return a pointer to the vtkKWLabelWithLabel, or NULL on error.
  virtual vtkKWLabelWithLabel* AddWidget(int id);

  // Description:
  // Get a vtkKWLabelWithLabel from the set, given its unique id.
  // Return a pointer to the vtkKWLabelWithLabel, or NULL on error.
  virtual vtkKWLabelWithLabel* GetWidget(int id);

protected:
  vtkKWLabelWithLabelSet() {};
  ~vtkKWLabelWithLabelSet() {};

  // Helper methods

  virtual vtkKWWidget* AllocateAndCreateWidget();

private:
  vtkKWLabelWithLabelSet(const vtkKWLabelWithLabelSet&); // Not implemented
  void operator=(const vtkKWLabelWithLabelSet&); // Not implemented
};

#endif
