/*=========================================================================

  Module:    $RCSfile: vtkKWWidgetWithLabelSubclass.h.in,v $

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkKWCheckButtonSetWithLabel - a vtkKWCheckButtonSet associated to a vtkKWLabel
// .SECTION Description
// This class creates a composite widget associating a vtkKWCheckButtonSet to a
// vtkKWLabel.
// As a subclass of vtkKWWidgetWithLabel, it inherits a label and methods to
// set its position and visibility. Note that the default label position 
// implemented in this class is on the left of the vtkKWCheckButtonSet, but 
// specific positions listed in vtkKWWidgetWithLabel are supported as well.
// .SECTION See Also
// vtkKWWidgetWithLabel vtkKWCheckButtonSet

#ifndef __vtkKWCheckButtonSetWithLabel_h
#define __vtkKWCheckButtonSetWithLabel_h

#include "vtkKWWidgetWithLabel.h"

class vtkKWApplication;
class vtkKWCheckButtonSet;

class KWWIDGETS_EXPORT vtkKWCheckButtonSetWithLabel : public vtkKWWidgetWithLabel
{
public:
  static vtkKWCheckButtonSetWithLabel* New();
  vtkTypeRevisionMacro(vtkKWCheckButtonSetWithLabel, vtkKWWidgetWithLabel);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Create the widget
  virtual void Create(vtkKWApplication *app);

  // Description:
  // Get the vtkKWCheckButtonSet that is being labeled
  virtual vtkKWCheckButtonSet* GetWidget();
  
  // Description:
  // Set the layout to allow the widget that is labeled to expand
  // horizontally automatically 
  virtual void SetExpandWidget(int);
  vtkBooleanMacro(ExpandWidget, int);
  vtkGetMacro(ExpandWidget, int);

  // Description:
  // Set the string that enables balloon help for this widget.
  // Override to pass down to children.
  virtual void SetBalloonHelpString(const char *str);

  // Description:
  // Update the "enable" state of the object and its internal parts.
  // Depending on different Ivars (this->Enabled, the application's 
  // Limited Edition Mode, etc.), the "enable" state of the object is updated
  // and propagated to its internal parts/subwidgets. This will, for example,
  // enable/disable parts of the widget UI, enable/disable the visibility
  // of 3D widgets, etc.
  virtual void UpdateEnableState();

protected:
  vtkKWCheckButtonSetWithLabel();
  ~vtkKWCheckButtonSetWithLabel();

  // Description:
  // Expand the widget that is being labeled
  int ExpandWidget;

  // Description:
  // Pack or repack the widget. To be implemented by subclasses.
  virtual void Pack();

  // Description:
  // Internal widget
  vtkKWCheckButtonSet *Widget;

private:

  vtkKWCheckButtonSetWithLabel(const vtkKWCheckButtonSetWithLabel&); // Not implemented
  void operator=(const vtkKWCheckButtonSetWithLabel&); // Not implemented
};

#endif
