/*=========================================================================

  Module:    $RCSfile: vtkKWWidgetWithSpinButtonsSubclass.cxx.in,v $

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#include "vtkKWMenuButtonWithSpinButtons.h"

#include "vtkKWSpinButtons.h"
#include "vtkObjectFactory.h"
#include "vtkKWMenuButton.h"

//----------------------------------------------------------------------------
vtkStandardNewMacro(vtkKWMenuButtonWithSpinButtons);
vtkCxxRevisionMacro(vtkKWMenuButtonWithSpinButtons, "$Revision: 1.3 $");

//----------------------------------------------------------------------------
vtkKWMenuButtonWithSpinButtons::vtkKWMenuButtonWithSpinButtons()
{
  this->Widget = vtkKWMenuButton::New();
}

//----------------------------------------------------------------------------
vtkKWMenuButtonWithSpinButtons::~vtkKWMenuButtonWithSpinButtons()
{
  if (this->Widget)
    {
    this->Widget->Delete();
    this->Widget = NULL;
    }
}

//----------------------------------------------------------------------------
vtkKWMenuButton* vtkKWMenuButtonWithSpinButtons::GetWidget()
{
  return this->Widget;
}

//----------------------------------------------------------------------------
void vtkKWMenuButtonWithSpinButtons::Create(vtkKWApplication *app)
{
  // Check if already created

  if (this->IsCreated())
    {
    vtkErrorMacro("vtkKWMenuButtonWithSpinButtons already created");
    return;
    }

  // Call the superclass

  this->Superclass::Create(app);

  // Create the widget

  if (!this->Widget->GetParent())
    {
    this->Widget->SetParent(this);
    }
  this->Widget->Create(app);
  if (!this->Widget->IsCreated())
    {
    vtkErrorMacro(<< this->GetClassName() << " failed creating vtkKWMenuButton");
    return;
    }

  // Pack

  this->Pack();

  // Update enable state
  
  this->UpdateEnableState();
}

//----------------------------------------------------------------------------
void vtkKWMenuButtonWithSpinButtons::Pack()
{
  if (!this->IsCreated())
    {
    return;
    }

  // Unpack everything

  if (this->Widget)
    {
    this->Widget->UnpackSiblings();
    }

  // Repack everything

  ostrstream tk_cmd;

  if (this->Widget && this->Widget->IsCreated())
    {
    tk_cmd << "pack " << this->Widget->GetWidgetName() 
           << " -anchor nw -side left -fill both -expand y" << endl;
    }

  if (this->SpinButtons && this->SpinButtons->IsCreated())
    {
    tk_cmd << "pack " << this->SpinButtons->GetWidgetName() 
           << " -anchor ne -side right -fill y -expand n" << endl;
    }
  
  tk_cmd << ends;
  this->Script(tk_cmd.str());
  tk_cmd.rdbuf()->freeze(0);
}

//----------------------------------------------------------------------------
void vtkKWMenuButtonWithSpinButtons::NextValueCallback()
{
  if (this->Widget)
    {
    this->Widget->NextValue();
    }
}

//----------------------------------------------------------------------------
void vtkKWMenuButtonWithSpinButtons::PreviousValueCallback()
{
  if (this->Widget)
    {
    this->Widget->PreviousValue();
    }
}

//----------------------------------------------------------------------------
void vtkKWMenuButtonWithSpinButtons::UpdateEnableState()
{
  this->Superclass::UpdateEnableState();
  
  this->PropagateEnableState(this->Widget);
}

// ---------------------------------------------------------------------------
void vtkKWMenuButtonWithSpinButtons::SetBalloonHelpString(const char *string)
{
  this->Superclass::SetBalloonHelpString(string);

  if (this->Widget)
    {
    this->Widget->SetBalloonHelpString(string);
    }
}

//----------------------------------------------------------------------------
void vtkKWMenuButtonWithSpinButtons::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

  os << indent << "Widget: ";
  if (this->Widget)
    {
    os << endl;
    this->Widget->PrintSelf(os, indent.GetNextIndent());
    }
  else
    {
    os << "None" << endl;
    }
}
