/*=========================================================================

  Module:    $RCSfile: vtkKWWidgetWithScrollbarsSubclass.cxx.in,v $

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#include "vtkKWApplication.h"
#include "vtkKWTextWithScrollbars.h"
#include "vtkObjectFactory.h"
#include "vtkKWScrollbar.h"
#include "vtkKWText.h"

#include <vtksys/stl/string>

//----------------------------------------------------------------------------
vtkStandardNewMacro(vtkKWTextWithScrollbars);
vtkCxxRevisionMacro(vtkKWTextWithScrollbars, "$Revision: 1.3 $");

//----------------------------------------------------------------------------
vtkKWTextWithScrollbars::vtkKWTextWithScrollbars()
{
  this->Widget = vtkKWText::New();
}

//----------------------------------------------------------------------------
vtkKWTextWithScrollbars::~vtkKWTextWithScrollbars()
{
  if (this->Widget)
    {
    this->Widget->Delete();
    this->Widget = NULL;
    }
}

//----------------------------------------------------------------------------
vtkKWText* vtkKWTextWithScrollbars::GetWidget()
{
  return this->Widget;
}

//----------------------------------------------------------------------------
void vtkKWTextWithScrollbars::Create(vtkKWApplication *app)
{
  // Check if already created

  if (this->IsCreated())
    {
    vtkErrorMacro(<< this->GetClassName() << " already created");
    return;
    }

  // Call the superclass to create the whole widget

  this->Superclass::Create(app);

  // Create the widget

  this->Widget->SetParent(this);
  this->Widget->Create(app);
  if (!this->Widget->IsCreated())
    {
    vtkErrorMacro(<< this->GetClassName() << " failed creating vtkKWText");
    return;
    }

  // Now that we have been created, make sure we are associated to
  // the scrollbars created by the super, *if any*

  this->AssociateVerticalScrollbarToWidget(this->Widget);
  this->AssociateHorizontalScrollbarToWidget(this->Widget);

  // Pack

  this->Pack();

  // Update enable state

  this->UpdateEnableState();
}

//----------------------------------------------------------------------------
void vtkKWTextWithScrollbars::CreateVerticalScrollbar(vtkKWApplication *app)
{
  this->Superclass::CreateVerticalScrollbar(app);
  this->AssociateVerticalScrollbarToWidget(this->Widget);
}

//----------------------------------------------------------------------------
void vtkKWTextWithScrollbars::CreateHorizontalScrollbar(vtkKWApplication *app)
{
  this->Superclass::CreateHorizontalScrollbar(app);
  this->AssociateHorizontalScrollbarToWidget(this->Widget);
}

//----------------------------------------------------------------------------
void vtkKWTextWithScrollbars::Pack()
{
  this->PackScrollbarsWithWidget(this->Widget);
}

//----------------------------------------------------------------------------
void vtkKWTextWithScrollbars::UpdateEnableState()
{
  this->Superclass::UpdateEnableState();

  this->PropagateEnableState(this->Widget);
}

//----------------------------------------------------------------------------
void vtkKWTextWithScrollbars::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
  os << indent << "Widget: ";
  if (this->Widget)
    {
    os << this->Widget << endl;
    }
  else
    {
    os << "(None)" << endl;
    }
}
